@@ -1,16 +1,34 @@
 #!/bin/sh
 
-JAVA_HOME=/usr/local/java
+JAVA_HOME=/usr/local/java5
 CLASSPATH=../build:.
-PROPERTIES=../../user1.prp
+PROPERTIES=../../user2.prp
 RUN="${JAVA_HOME}/bin/java -cp ${CLASSPATH} -Djcifs.properties=${PROPERTIES}"
 
-SERVER=dc1.w.net
-SHARE=tmp
-WRITE_DIR=test/
-SRC_DIR=test/Junk
-FILE1=test/Junk/10883563.doc
+#SERVER=192.168.2.110
+#SERVER=dc1.w.net
+#SHARE=tmp
+#DIR=test
 
+# Domain-based DFS
+#SERVER=192.168.2.110
+SERVER=w.net
+SHARE=root2
+DIR=test
+
+# smb://fs1.w.net/DFSStandaloneRoot/DFSStandaloneLink/test/
+# smb://dc1.w.net/root2/link2/test/
+# smb://dc1.w.net/tmp/test/
+# smb://dc3.x.net/tmp/test/
+# Stand-alone DFS
+#SERVER=192.168.2.113
+#SERVER=fs1.w.net
+#SHARE=DFSStandaloneRoot
+#DIR=DFSStandaloneLink/test
+
+WRITE_DIR=${DIR}/
+SRC_DIR=${DIR}/Junk
+FILE1=${DIR}/Junk/10883563.doc
 URL_SERVER=smb://${SERVER}/
 URL_SHARE=${URL_SERVER}${SHARE}/
 URL_WRITE_DIR=${URL_SHARE}${WRITE_DIR}
@@ -27,7 +45,7 @@ $RUN GetSecurity ${URL_SHARE}
 $RUN GetShareSecurity ${URL_WRITE_DIR}
 $RUN SidCrawler ${URL_WRITE_DIR} 5
 $RUN GetGroupMemberSidsFromURL ${URL_WRITE_DIR}
-$RUN InterruptTest ${URL_WRITE_DIR}${FILE1}
+$RUN InterruptTest ${URL_SHARE}${FILE1}
 $RUN AllocInfo ${URL_SHARE}
 $RUN Append ${URL_WRITE_DIR}Append.txt
 $RUN AuthListFiles smb://bogus\@${SERVER}/${SHARE}/
