@@ -24,7 +24,7 @@ import java.util.Iterator;
 
 import jcifs.dcerpc.msrpc.*;
 
-class DcerpcBinding {
+public class DcerpcBinding {
 
     private static HashMap INTERFACES;
 
@@ -36,6 +36,11 @@ class DcerpcBinding {
         INTERFACES.put("netdfs", netdfs.getSyntax());
     }
 
+    public static void addInterface(String name, String syntax)
+    {
+        INTERFACES.put(name, syntax);
+    }
+
     String proto;
     String server;
     String endpoint = null;
