@@ -106,6 +106,7 @@ public abstract class DcerpcHandle implements DcerpcConstants {
     protected int max_xmit = 4280;
     protected int max_recv = max_xmit;
     protected int state = 0;
+    protected DcerpcSecurityProvider securityProvider = null;
     private static int call_id = 1;
 
     public static DcerpcHandle getHandle(String url,
@@ -117,6 +118,16 @@ public abstract class DcerpcHandle implements DcerpcConstants {
         throw new DcerpcException("DCERPC transport not supported: " + url);
     }
 
+    public void bind() throws DcerpcException, IOException {
+        try {
+            state = 1;
+            DcerpcMessage bind = new DcerpcBind(binding, this);
+            sendrecv(bind);
+        } catch (IOException ioe) {
+            state = 0;
+            throw ioe;
+        }
+    }
     public void sendrecv(DcerpcMessage msg) throws DcerpcException, IOException {
         byte[] stub, frag;
         NdrBuffer buf, fbuf;
@@ -124,9 +135,7 @@ public abstract class DcerpcHandle implements DcerpcConstants {
         DcerpcException de;
 
         if (state == 0) {
-            state = 1;
-            DcerpcMessage bind = new DcerpcBind(binding, this);
-            sendrecv(bind);
+            bind();
         }
 
         isDirect = msg instanceof DcerpcBind;
@@ -146,8 +155,14 @@ public abstract class DcerpcHandle implements DcerpcConstants {
 
             msg.encode(buf);
 
+            if (securityProvider != null) {
+                buf.setIndex(0);
+                securityProvider.wrap(buf);
+            }
+
             tot = buf.getLength();
             off = 0;
+
             while (off < tot) {
                 msg.call_id = call_id++;
 
@@ -171,6 +186,14 @@ public abstract class DcerpcHandle implements DcerpcConstants {
 
             doReceiveFragment(stub, isDirect);
             buf.reset();
+            buf.setIndex(8);
+            buf.setLength(buf.dec_ndr_short());
+
+            if (securityProvider != null)
+                securityProvider.unwrap(buf);
+
+            buf.setIndex(0);
+
             msg.decode_header(buf);
 
             off = 24;
@@ -188,6 +211,13 @@ public abstract class DcerpcHandle implements DcerpcConstants {
                 }
 
                 doReceiveFragment(frag, isDirect);
+                fbuf.reset();
+                fbuf.setIndex(8);
+                fbuf.setLength(fbuf.dec_ndr_short());
+
+                if (securityProvider != null)
+                    securityProvider.unwrap(fbuf);
+
                 fbuf.reset();
                 msg.decode_header(fbuf);
                 stub_frag_len = msg.length - 24;
@@ -203,7 +233,7 @@ public abstract class DcerpcHandle implements DcerpcConstants {
                 off += stub_frag_len;
             }
 
-            buf.reset();
+            buf = new NdrBuffer(stub, 0);
             msg.decode(buf);
         } finally {
             jcifs.smb.BufferCache.releaseBuffer(stub);
@@ -213,6 +243,10 @@ public abstract class DcerpcHandle implements DcerpcConstants {
             throw de;
     }
 
+    public void setDcerpcSecurityProvider(DcerpcSecurityProvider securityProvider)
+    {
+        this.securityProvider = securityProvider;
+    }
     public String getServer() {
         if (this instanceof DcerpcPipeHandle)
             return ((DcerpcPipeHandle)this).pipe.getServer();
