@@ -48,26 +48,20 @@ public class BufferCache {
         return buf;
     }
 
-    static void getBuffers( SmbComTransaction req, SmbComTransactionResponse rsp ) {
+    static void getBuffers( SmbComTransaction req,
+                    SmbComTransactionResponse rsp ) throws InterruptedException {
         synchronized( cache ) {
-            try {
-                while ((freeBuffers + (MAX_BUFFERS - numBuffers)) < 2) {
-                    cache.wait();
-                }
-                req.txn_buf = getBuffer0();
-                rsp.txn_buf = getBuffer0();
-            } catch( InterruptedException ie ) {
-                ie.printStackTrace();
+            while ((freeBuffers + (MAX_BUFFERS - numBuffers)) < 2) {
+                cache.wait();
             }
+            req.txn_buf = getBuffer0();
+            rsp.txn_buf = getBuffer0();
         }
     }
-    static public byte[] getBuffer() {
+    static public byte[] getBuffer() throws InterruptedException {
         synchronized( cache ) {
             while ((freeBuffers + (MAX_BUFFERS - numBuffers)) < 1) {
-                try {
-                    cache.wait();
-                } catch(InterruptedException ie) {
-                }
+                cache.wait();
             }
             return getBuffer0();
         }
