@@ -242,6 +242,19 @@ public class SID extends rpc.sid_t {
             }
         }
     }
+    public static byte[] toByteArray(rpc.sid_t sid) {
+        byte[] dst = new byte[1 + 1 + 6 + sid.sub_authority_count * 4];
+        int di = 0;
+        dst[di++] = sid.revision;
+        dst[di++] = sid.sub_authority_count;
+        System.arraycopy(sid.identifier_authority, 0, dst, di, 6);
+        di += 6;
+        for (int ii = 0; ii < sid.sub_authority_count; ii++) {
+            jcifs.util.Encdec.enc_uint32le(sid.sub_authority[ii], dst, di);
+            di += 4;
+        }
+        return dst;
+    }
 
     int type;
     String domainName = null;
@@ -315,7 +328,7 @@ public class SID extends rpc.sid_t {
         }
         this.sub_authority[i] = rid;
     }
-    SID(rpc.sid_t sid,
+    public SID(rpc.sid_t sid,
                     int type,
                     String domainName,
                     String acctName,
