@@ -33,7 +33,7 @@ public class SmbRandomAccessFile implements DataOutput, DataInput {
 
     private SmbFile file;
     private long fp;
-    private int openFlags, readSize, writeSize, ch, options = 0;
+    private int openFlags, access = 0, readSize, writeSize, ch, options = 0;
     private byte[] tmp = new byte[8];
     private SmbComWriteAndXResponse write_andx_resp = null;
 
@@ -50,10 +50,11 @@ public class SmbRandomAccessFile implements DataOutput, DataInput {
             this.openFlags = SmbFile.O_CREAT | SmbFile.O_RDWR | SmbFile.O_APPEND;
             write_andx_resp = new SmbComWriteAndXResponse();
             options = WRITE_OPTIONS;
+            access = SmbConstants.FILE_READ_DATA | SmbConstants.FILE_WRITE_DATA;
         } else {
             throw new IllegalArgumentException( "Invalid mode" );
         }
-        file.open( openFlags, SmbFile.ATTR_NORMAL, options );
+        file.open( openFlags, access, SmbFile.ATTR_NORMAL, options );
         readSize = file.tree.session.transport.rcv_buf_size - 70;
         writeSize = file.tree.session.transport.snd_buf_size - 70;
         fp = 0L;
@@ -76,7 +77,7 @@ public class SmbRandomAccessFile implements DataOutput, DataInput {
 
         // ensure file is open
         if( file.isOpen() == false ) {
-            file.open( openFlags, SmbFile.ATTR_NORMAL, options );
+            file.open( openFlags, 0, SmbFile.ATTR_NORMAL, options );
         }
 
         int r, n;
@@ -129,7 +130,7 @@ public class SmbRandomAccessFile implements DataOutput, DataInput {
 
         // ensure file is open
         if( file.isOpen() == false ) {
-            file.open( openFlags, SmbFile.ATTR_NORMAL, options );
+            file.open( openFlags, 0, SmbFile.ATTR_NORMAL, options );
         }
 
         int w;
@@ -153,7 +154,7 @@ public class SmbRandomAccessFile implements DataOutput, DataInput {
     public void setLength( long newLength ) throws SmbException {
         // ensure file is open
         if( file.isOpen() == false ) {
-            file.open( openFlags, SmbFile.ATTR_NORMAL, options );
+            file.open( openFlags, 0, SmbFile.ATTR_NORMAL, options );
         }
         SmbComWriteResponse rsp = new SmbComWriteResponse();
         file.send( new SmbComWrite( file.fid, (int)(newLength & 0xFFFFFFFFL), 0, tmp, 0, 0 ), rsp );
