@@ -22,32 +22,14 @@ import java.util.Vector;
 import java.util.Enumeration;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
+import java.io.IOException;
 import jcifs.Config;
 import jcifs.UniAddress;
 import jcifs.netbios.NbtAddress;
+import jcifs.util.MD4;
 
 /**
- * The class represents a user's session established with an SMB/CIFS
- * server. This class is used internally to the jCIFS library however
- * applications may wish to authenticate aribrary user credentials
- * with the <tt>logon</tt> method. It is noteworthy that jCIFS does not
- * support DCE/RPC at this time and therefore does not use the NETLOGON
- * procedure. Instead, it simply performs a "tree connect" to IPC$ using
- * the supplied credentials. This is only a subset of the NETLOGON procedure
- * but is achives the same effect.
-
-Note that it is possible to change the resource against which clients
-are authenticated to be something other than <tt>IPC$</tt> using the
-<tt>jcifs.smb.client.logonShare</tt> property. This can be used to
-provide simple group based access control. For example, one could setup
-the NTLM HTTP Filter with the <tt>jcifs.smb.client.domainController</tt>
-init parameter set to the name of the server used for authentication. On
-that host, create a share called JCIFSAUTH and adjust the access control
-list for that share to permit only the clients that should have access to
-the target website. Finally, set the <tt>jcifs.smb.client.logonShare</tt>
-to JCIFSAUTH. This should restrict access to only those clients that have
-access to the JCIFSAUTH share. The access control on that share can be
-changed without changing init parameters or reinitializing the webapp.
+ * Update June 2009: This logon method of this class does not and never will support NTLMv2. JCIFS does not implement the acceptor side of NTLM authentication. It can only initiate NTLM authentication as a client.
  */
 
 public final class SmbSession {
@@ -154,7 +136,9 @@ do {
  * occurs an <tt>SmbException</tt> will be thrown. If the credentials are
  * valid, the method will return without throwing an exception. See the
  * last <a href="../../../faq.html">FAQ</a> question.
- *
+ * <p>
+ * Update June 2009: This method does not support NTLMv2. JCIFS does not implement the acceptor side of NTLM authentication. It can only initiate NTLM authentication as a client.
+ * <p>
  * See also the <tt>jcifs.smb.client.logonShare</tt> property.
  */
     public static void logon( UniAddress dc,
@@ -218,7 +202,7 @@ do {
     }
     synchronized SmbTransport transport() {
         if( transport == null ) {
-            transport = SmbTransport.getSmbTransport( address, port, localAddr, localPort );
+            transport = SmbTransport.getSmbTransport( address, port, localAddr, localPort, null );
         }
         return transport;
     }
@@ -249,7 +233,12 @@ do {
     }
     void sessionSetup( ServerMessageBlock andx,
                             ServerMessageBlock andxResponse ) throws SmbException {
+        NtlmContext nctx = null;
         SmbException ex = null;
+        SmbComSessionSetupAndX request;
+        SmbComSessionSetupAndXResponse response;
+        byte[] token = new byte[0];
+        int state = 10;
 
 synchronized( transport() ) {
         if( sessionSetup ) {
@@ -265,48 +254,145 @@ synchronized( transport() ) {
         if( transport.log.level >= 4 )
             transport.log.println( "sessionSetup: accountName=" + auth.username + ",primaryDomain=" + auth.domain );
 
-        SmbComSessionSetupAndX request = new SmbComSessionSetupAndX( this, andx );
-        SmbComSessionSetupAndXResponse response = new SmbComSessionSetupAndXResponse( andxResponse );
-
-        /* Create SMB signature digest if necessary
-         * Only the first SMB_COM_SESSION_SETUP_ANX with non-null or
-         * blank password initializes signing.
+        /* We explicitly set uid to 0 here to prevent a new
+         * SMB_COM_SESSION_SETUP_ANDX from having it's uid set to an
+         * old value when the session is re-established. Otherwise a
+         * "The parameter is incorrect" error can occur.
          */
-        if (transport.isSignatureSetupRequired( auth )) {
-            if( auth.hashesExternal && NtlmPasswordAuthentication.DEFAULT_PASSWORD != NtlmPasswordAuthentication.BLANK ) {
-                /* preauthentication
-                 */
-                transport.getSmbSession( NtlmPasswordAuthentication.DEFAULT ).getSmbTree( LOGON_SHARE, null ).treeConnect( null, null );
-            } else {
-                request.digest = new SigningDigest( transport, auth );
+        uid = 0;
+
+        do {
+            switch (state) {
+                case 10: /* NTLM */
+                    if (auth != NtlmPasswordAuthentication.ANONYMOUS &&
+                            transport.hasCapability(SmbConstants.CAP_EXTENDED_SECURITY)) {
+                        state = 20; /* NTLMSSP */
+                        break;
+                    }
+
+                    request = new SmbComSessionSetupAndX( this, andx, auth );
+                    response = new SmbComSessionSetupAndXResponse( andxResponse );
+
+                    /* Create SMB signature digest if necessary
+                     * Only the first SMB_COM_SESSION_SETUP_ANX with non-null or
+                     * blank password initializes signing.
+                     */
+                    if (transport.isSignatureSetupRequired( auth )) {
+                        if( auth.hashesExternal && NtlmPasswordAuthentication.DEFAULT_PASSWORD != NtlmPasswordAuthentication.BLANK ) {
+                            /* preauthentication
+                             */
+                            transport.getSmbSession( NtlmPasswordAuthentication.DEFAULT ).getSmbTree( LOGON_SHARE, null ).treeConnect( null, null );
+                        } else {
+                            byte[] signingKey = auth.getSigningKey(transport.server.encryptionKey);
+                            request.digest = new SigningDigest(signingKey, false);
+                        }
+                    }
+
+                    request.auth = auth;
+
+                    try {
+                        transport.send( request, response );
+                    } catch (SmbAuthException sae) {
+                        throw sae;
+                    } catch (SmbException se) {
+                        ex = se;
+                    }
+
+                    if( response.isLoggedInAsGuest &&
+                                "GUEST".equalsIgnoreCase( auth.username ) == false &&
+                                transport.server.security != SmbConstants.SECURITY_SHARE) {
+                        throw new SmbAuthException( NtStatus.NT_STATUS_LOGON_FAILURE );
+                    }
+
+                    if (ex != null)
+                        throw ex;
+
+                    uid = response.uid;
+
+                    if( request.digest != null ) {
+                        /* success - install the signing digest */
+                        transport.digest = request.digest;
+                    }
+
+                    sessionSetup = true;
+                    state = 0;
+
+                    break;
+                case 20:
+                    if (nctx == null) {
+                        boolean doSigning = (transport.flags2 & ServerMessageBlock.FLAGS2_SECURITY_SIGNATURES) != 0;
+                        nctx = new NtlmContext(auth, doSigning);
+                    }
+
+                    if (nctx.isEstablished()) {
+                        sessionSetup = true;
+                        state = 0;
+                        break;
+                    }
+
+                    try {
+                        token = nctx.initSecContext(token, 0, token.length);
+                    } catch (SmbException se) {
+                        /* We must close the transport or the server will be expecting a
+                         * Type3Message. Otherwise, when we send a Type1Message it will return
+                         * "Invalid parameter".
+                         */
+                        try { transport.disconnect(true); } catch (IOException ioe) {}
+                        uid = 0;
+                        throw se;
+                    }
+
+                    if (token != null) {
+                        request = new SmbComSessionSetupAndX(this, null, token);
+                        response = new SmbComSessionSetupAndXResponse(null);
+
+                        if (transport.isSignatureSetupRequired( auth )) {
+                            byte[] signingKey = nctx.getSigningKey();
+                            if (signingKey != null)
+                                request.digest = new SigningDigest(signingKey, true);
+                        }
+
+                        request.uid = uid;
+                        uid = 0;
+
+                        try {
+                            transport.send( request, response );
+                        } catch (SmbAuthException sae) {
+                            throw sae;
+                        } catch (SmbException se) {
+                            ex = se;
+                            /* Apparently once a successfull NTLMSSP login occurs, the
+                             * server will return "Access denied" even if a logoff is
+                             * sent. Unfortunately calling disconnect() doesn't always
+                             * actually shutdown the connection before other threads
+                             * have committed themselves (e.g. InterruptTest example).
+                             */
+                            try { transport.disconnect(true); } catch (Exception e) {}
+                        }
+
+                        if( response.isLoggedInAsGuest &&
+                                    "GUEST".equalsIgnoreCase( auth.username ) == false) {
+                            throw new SmbAuthException( NtStatus.NT_STATUS_LOGON_FAILURE );
+                        }
+
+                        if (ex != null)
+                            throw ex;
+
+                        uid = response.uid;
+
+                        if (request.digest != null) {
+                            /* success - install the signing digest */
+                            transport.digest = request.digest;
+                        }
+
+                        token = response.blob;
+                    }
+
+                    break;
+                default:
+                    throw new SmbException("Unexpected session setup state: " + state);
             }
-        }
-
-        request.auth = auth;
-
-        try {
-            transport.send( request, response );
-        } catch (SmbAuthException sae) {
-            throw sae;
-        } catch (SmbException se) {
-            ex = se;
-        }
-
-        if( response.isLoggedInAsGuest &&
-                    "GUEST".equalsIgnoreCase( auth.username ) == false) {
-            throw new SmbAuthException( NtStatus.NT_STATUS_LOGON_FAILURE );
-        }
-
-        uid = response.uid;
-        sessionSetup = true;
-
-        if( request.digest != null ) {
-            /* success - install the signing digest */
-            transport.digest = request.digest;
-        }
-
-        if (ex != null)
-            throw ex;
+        } while (state != 0);
 }
     }
     void logoff( boolean inError ) {
@@ -321,17 +407,17 @@ synchronized( transport() ) {
         }
 
         if( !inError && transport.server.security != ServerMessageBlock.SECURITY_SHARE ) {
-
             /*
              * Logoff And X Request / Response
              */
-
+    
             SmbComLogoffAndX request = new SmbComLogoffAndX( null );
             request.uid = uid;
             try {
                 transport.send( request, null );
             } catch( SmbException se ) {
             }
+            uid = 0;
         }
 
         sessionSetup = false;
