@@ -38,10 +38,10 @@ public class SmbTransport extends Transport implements SmbConstants {
     static HashMap dfsRoots = null;
 
     static synchronized SmbTransport getSmbTransport( UniAddress address, int port ) {
-        return getSmbTransport( address, port, LADDR, LPORT );
+        return getSmbTransport( address, port, LADDR, LPORT, null );
     }
     static synchronized SmbTransport getSmbTransport( UniAddress address, int port,
-                                    InetAddress localAddr, int localPort ) {
+                                    InetAddress localAddr, int localPort, String hostName ) {
         SmbTransport conn;
 
         synchronized( CONNECTIONS ) {
@@ -49,7 +49,7 @@ public class SmbTransport extends Transport implements SmbConstants {
                 ListIterator iter = CONNECTIONS.listIterator();
                 while( iter.hasNext() ) {
                     conn = (SmbTransport)iter.next();
-                    if( conn.matches( address, port, localAddr, localPort ) &&
+                    if( conn.matches( address, port, localAddr, localPort, hostName ) &&
                             ( SSN_LIMIT == 0 || conn.sessions.size() < SSN_LIMIT )) {
                         return conn;
                     }
@@ -82,6 +82,7 @@ public class SmbTransport extends Transport implements SmbConstants {
         int serverTimeZone;
         int encryptionKeyLength;
         byte[] encryptionKey;
+        byte[] guid;
     }
 
     InetAddress localAddr;
@@ -106,7 +107,7 @@ public class SmbTransport extends Transport implements SmbConstants {
     int capabilities = CAPABILITIES;
     int sessionKey = 0x00000000;
     boolean useUnicode = USE_UNICODE;
-    String tconHostName;
+    String tconHostName = null;
 
     SmbTransport( UniAddress address, int port, InetAddress localAddr, int localPort ) {
         this.address = address;
@@ -149,8 +150,11 @@ public class SmbTransport extends Transport implements SmbConstants {
 
         return ssn;
     }
-    boolean matches( UniAddress address, int port, InetAddress localAddr, int localPort ) {
-        return address.equals( this.address ) &&
+    boolean matches( UniAddress address, int port, InetAddress localAddr, int localPort, String hostName ) {
+        if (hostName == null)
+            hostName = address.getHostName();
+        return (this.tconHostName == null || hostName.equalsIgnoreCase(this.tconHostName)) &&
+                    address.equals( this.address ) &&
                     (port == 0 || port == this.port ||
                             /* port 139 is ok if 445 was requested */
                             (port == 445 && this.port == 139)) &&
@@ -292,7 +296,7 @@ public class SmbTransport extends Transport implements SmbConstants {
         try {
             super.connect( RESPONSE_TIMEOUT );
         } catch( TransportException te ) {
-            throw new SmbException( te.getMessage(), te );
+            throw new SmbException( "Failed to connect: " + address, te );
         }
     }
     protected void doConnect() throws IOException {
@@ -314,8 +318,10 @@ public class SmbTransport extends Transport implements SmbConstants {
         if( resp.dialectIndex > 10 ) {
             throw new SmbException( "This client does not support the negotiated dialect." );
         }
-        if (server.encryptionKeyLength != 8 && LM_COMPATIBILITY == 0) {
-            throw new SmbException("Encryption key length is not 8 as expected. This could indicate that the server requires NTLMv2. JCIFS does not fully support NTLMv2 but you can try setting jcifs.smb.lmCompatibility = 3.");
+        if ((server.capabilities & CAP_EXTENDED_SECURITY) != CAP_EXTENDED_SECURITY &&
+                    server.encryptionKeyLength != 8 &&
+                    LM_COMPATIBILITY == 0) {
+            throw new SmbException("Unexpected encryption key length: " + server.encryptionKeyLength);
         }
 
         /* Adjust negotiated values */
@@ -330,6 +336,9 @@ public class SmbTransport extends Transport implements SmbConstants {
         if (maxMpxCount < 1) maxMpxCount = 1;
         snd_buf_size = Math.min( snd_buf_size, server.maxBufferSize );
         capabilities &= server.capabilities;
+        if ((server.capabilities & CAP_EXTENDED_SECURITY) == CAP_EXTENDED_SECURITY)
+            capabilities |= CAP_EXTENDED_SECURITY; // & doesn't copy high bit
+
         if ((capabilities & ServerMessageBlock.CAP_UNICODE) == 0) {
             // server doesn't want unicode
             if (FORCE_UNICODE) {
@@ -444,6 +453,7 @@ public class SmbTransport extends Transport implements SmbConstants {
     protected void doRecv( Response response ) throws IOException {
         ServerMessageBlock resp = (ServerMessageBlock)response;
         resp.useUnicode = useUnicode;
+        resp.extendedSecurity = (capabilities & CAP_EXTENDED_SECURITY) == CAP_EXTENDED_SECURITY;
 
         synchronized (BUF) {
             System.arraycopy( sbuf, 0, BUF, 0, 4 + HEADER_LENGTH );
@@ -518,11 +528,16 @@ public class SmbTransport extends Transport implements SmbConstants {
                     throw new SmbException( resp.errorCode, null );
                 }
 
-                DfsReferral[] drs = getDfsReferrals(req.auth, req.path, 1);
-                SmbFile.dfs.insert(req.path, drs[0]);
-                throw drs[0];
+                DfsReferral dr = getDfsReferrals(req.auth, req.path, 1);
+                if (dr == null)
+                    throw new SmbException(resp.errorCode, null);   
+
+                SmbFile.dfs.insert(req.path, dr);
+                throw dr;
             case 0x80000005:  /* STATUS_BUFFER_OVERFLOW */
                 break; /* normal for DCERPC named pipes */
+            case NtStatus.NT_STATUS_MORE_PROCESSING_REQUIRED:
+                break; /* normal for NTLMSSP */
             default:
                 throw new SmbException( resp.errorCode, null );
         }
@@ -661,7 +676,52 @@ public class SmbTransport extends Transport implements SmbConstants {
             result[ri++] = "";
         }
     }
-    DfsReferral[] getDfsReferrals(NtlmPasswordAuthentication auth,
+    DfsReferral getDfsReferrals(NtlmPasswordAuthentication auth,
+                String path,
+                int rn) throws SmbException {
+        SmbTree ipc = getSmbSession( auth ).getSmbTree( "IPC$", null );
+        Trans2GetDfsReferralResponse resp = new Trans2GetDfsReferralResponse();
+        ipc.send( new Trans2GetDfsReferral( path ), resp );
+
+        if (resp.numReferrals == 0) {
+            return null;
+        } else if (rn == 0 || resp.numReferrals < rn) {
+            rn = resp.numReferrals;
+        }
+
+        DfsReferral dr = new DfsReferral();
+
+        String[] arr = new String[4];
+        long expiration = System.currentTimeMillis() + Dfs.TTL * 1000;
+
+        int di = 0;
+        for ( ;; ) {
+                        /* NTLM HTTP Authentication must be re-negotiated
+                         * with challenge from 'server' to access DFS vol. */
+            dr.resolveHashes = auth.hashesExternal;
+            dr.ttl = resp.referrals[di].ttl;
+            dr.expiration = expiration;
+            if (path.equals("")) {
+                dr.server = resp.referrals[di].path.substring(1).toLowerCase();
+            } else {
+                dfsPathSplit(resp.referrals[di].node, arr);
+                dr.server = arr[1];
+                dr.share = arr[2];
+                dr.path = arr[3];
+            }
+            dr.pathConsumed = resp.pathConsumed;
+
+            di++;
+            if (di == rn)
+                break;
+
+            dr.append(new DfsReferral());
+            dr = dr.next;
+        }
+
+        return dr.next;
+    }
+    DfsReferral[] __getDfsReferrals(NtlmPasswordAuthentication auth,
                 String path,
                 int rn) throws SmbException {
         SmbTree ipc = getSmbSession( auth ).getSmbTree( "IPC$", null );
@@ -697,38 +757,39 @@ public class SmbTransport extends Transport implements SmbConstants {
 
         return drs;
     }
-    FileEntry[] getDfsRoots(String domainName, NtlmPasswordAuthentication auth) throws IOException {
-        MsrpcDfsRootEnum rpc;
-        DcerpcHandle handle = null;
 
-        /* Procedure:
-         * Lookup a DC in the target domain
-         * Ask the DC for a referral for the domain (e.g. "\example.com")
-         * Do NetrDfsEnumEx on the server returned in the referral to
-         * get roots in target domain
-         */
-
-        UniAddress dc = UniAddress.getByName(domainName);
-        SmbTransport trans = SmbTransport.getSmbTransport(dc, 0);
-        DfsReferral[] dr = trans.getDfsReferrals(auth, "\\" + domainName, 1);
-
-        handle = DcerpcHandle.getHandle("ncacn_np:" +
-                    UniAddress.getByName(dr[0].server).getHostAddress() +
-                    "[\\PIPE\\netdfs]", auth);
-        try {
-            rpc = new MsrpcDfsRootEnum(domainName);
-            handle.sendrecv(rpc);
-            if (rpc.retval != 0)
-                throw new SmbException(rpc.retval, true);
-            return rpc.getEntries();
-        } finally {
-            try {
-                handle.close();
-            } catch(IOException ioe) {
-                if (log.level >= 4)
-                    ioe.printStackTrace(log);
-            }
-        }
-    }
+//    FileEntry[] getDfsRoots(String domainName, NtlmPasswordAuthentication auth) throws IOException {
+//        MsrpcDfsRootEnum rpc;
+//        DcerpcHandle handle = null;
+//
+//        /* Procedure:
+//         * Lookup a DC in the target domain
+//         * Ask the DC for a referral for the domain (e.g. "\example.com")
+//         * Do NetrDfsEnumEx on the server returned in the referral to
+//         * get roots in target domain
+//         */
+//
+//        UniAddress dc = UniAddress.getByName(domainName);
+//        SmbTransport trans = SmbTransport.getSmbTransport(dc, 0);
+//        DfsReferral[] dr = trans.getDfsReferrals(auth, "\\" + domainName, 1);
+//
+//        handle = DcerpcHandle.getHandle("ncacn_np:" +
+//                    UniAddress.getByName(dr[0].server).getHostAddress() +
+//                    "[\\PIPE\\netdfs]", auth);
+//        try {
+//            rpc = new MsrpcDfsRootEnum(domainName);
+//            handle.sendrecv(rpc);
+//            if (rpc.retval != 0)
+//                throw new SmbException(rpc.retval, true);
+//            return rpc.getEntries();
+//        } finally {
+//            try {
+//                handle.close();
+//            } catch(IOException ioe) {
+//                if (log.level >= 4)
+//                    ioe.printStackTrace(log);
+//            }
+//        }
+//    }
 }
 
